// Nazev:   Implementace parseru VHDL
// Soubor:  lex.CPP
// Popis:   implementace lexikalniho analyzatoru
// Datum:   2006-05-10
// Autor:   strakam@fit.vutbr.cz, Martin Straka, DIT 2006
//========================================================================
#include "parser.h"
#include "lex.h"
#include <stdio.h>
#include <fstream>
#include <dirent.h>
#include <assert.h>
#include <iostream>
#include <list>
using namespace std;

typedef struct{string name;} Tsource;

list<Tsource> SourceListVHDL;
list<Tsource>::iterator it_SourceListVHDL;
Tsource SourceVHDL;

void add_vhdl(string name)
{             SourceVHDL.name=name;
              SourceListVHDL.push_back(SourceVHDL);
}


//-----------------------------------------------------------------------------
void GenerateFT(string fn_in, short arch){
     if (lexInit(fn_in)) {
        parse(arch);
        lexFree(fn_in);
    }
}


//-----------------------------------------------------------------------------
void GenerateFTCH(string fn_in, string fn_inch, short arch){
     if (lexInit(fn_in)) {
        parse(arch);
        lexFree(fn_in);
    }
//       if (lexInit2(fn_inch)) {
 //       parse(arch);
//        lexFree2(fn_inch);
 //   }  
    
}



//-----------------------------------------------------------------------------
void PrintInfo(string file)
{
  cout << "\nThis program generate different types of FT architectures\n";
  cout << "based on TMR or duplex from input VHDL file. The modules\n";
  cout << "of architectures are generated as PRMs...\n\n";
  cout << "Syntax for FT architectures WITHOUT checkers:\n";
  cout << "---------------------------------------------\n";  
  cout << "> " << file << " [parameter] <source.vhd>\n\n";
  cout << "   Parameters:\n";
  cout << "      -tmr0   ... for simple TMR architecture\n";
  cout << "      -tmr1   ... for 3FU-3V architecture\n";
  cout << "      -tmr2   ... for TMR-2V-CMP architecture\n";  
  cout << "      -dpl0   ... for simple Duplex architecture\n";
  cout << "      -dpl4   ... for 3FU-2MUX architecture\n";
  cout << "      -dpl5   ... for 3FU-1MUX architecture\n";
  cout << "      -dpl6   ... for 3FU-2MUX-CMP architecture\n";    
  cout << "\n";   
  cout << "Syntax for FT architectures WITH checkers:\n";
  cout << "------------------------------------------\n";
  cout << "> " << file << " [parameter] <source.vhd> <checker.vhd>\n\n";
  cout << "   Parameters:\n";
  cout << "      -fuch   ... for FU with checker only\n";
  cout << "      -tmr3   ... for TMR+3CH architecture\n";
  cout << "      -dpl2   ... for DUPLEX+2CH architecture\n";  
  cout << "      -dpl1   ... for DUPLEX+1CH architecture\n"; 
}

//---------------------------------------------------------------------------
int main(int argc, char* argv[]) {

  string params;

  if (argc==3) {    
    params=argv[1];
            if (params=="-tmr0") { 
                  GenerateFT(argv[2],_TMR);
            }
            else if (params=="-tmr1") {
                 GenerateFT(argv[2],_3FU_3V); 
            }
            else if (params=="-tmr2") {
                 GenerateFT(argv[2],_TMR_2V_CMP); 
            }
            else if (params=="-dpl4") {
                 GenerateFT(argv[2],_3FU_2MUX_); 
            }    
            else if (params=="-dpl5") {
                 GenerateFT(argv[2],_3FU_1MUX); 
            }                     
            else if (params=="-dpl6") {
                 GenerateFT(argv[2],_3FU_2MUX_CMP); 
            }            
            else if (params=="-dpl0") {
                 GenerateFT(argv[2],_DUPL_CMP); 
            }                                    
            else PrintInfo(argv[0]);         
   }    
   else if (argc==4) {
        params=argv[1];
             if (params=="-fuch") { 
                  GenerateFTCH(argv[2],argv[3],_FUCH);
            }
            else if (params=="-tmr3") {
                 GenerateFTCH(argv[2],argv[3],_TMR_3CH); 
            }
            else if (params=="-dpl1") {
                 GenerateFTCH(argv[2],argv[3],_DLP_1CH); 
            }    
            else if (params=="-dpl2") {
                 GenerateFTCH(argv[2],argv[3],_DLP_2CH); 
            }                                    
            else PrintInfo(argv[0]);                
   }
   else PrintInfo(argv[0]);
}
